from django.conf import settings
from rest_framework import serializers
from arkindex_common.ml_tool import MLToolType, MLTool
from arkindex.project.serializer_fields import EnumField


class MLToolLightSerializer(serializers.Serializer):
    type = EnumField(MLToolType)
    slug = serializers.SlugField()

    def validate(self, data):
        try:
            return MLTool.get(settings.ML_CLASSIFIERS_DIR, data['type'], data['slug'])
        except ValueError as e:
            raise serializers.ValidationError(str(e))


class MLToolSerializer(MLToolLightSerializer):
    """
    Serialize a machine learning tool for display
    """
    name = serializers.CharField()
    version = serializers.CharField()


class MLToolTaskListSerializer(serializers.ListSerializer):

    def validate(self, data):
        seen = set()
        for ml_tool in data:
            if ml_tool in seen:
                raise serializers.ValidationError(['Tool {} is duplicated'.format(ml_tool)])
            seen.add(ml_tool)

            for parent in ml_tool.parents:
                if parent not in data:
                    raise serializers.ValidationError({
                        'parents': ['Missing dependency {} for tool {}'.format(parent, ml_tool)]
                    })
        return data


class MLToolTaskSerializer(MLToolLightSerializer):
    parents = MLToolLightSerializer(many=True, default=list)

    class Meta:
        list_serializer_class = MLToolTaskListSerializer

    def validate(self, data):
        ml_tool = super().validate(data)
        ml_tool.parents = data['parents']
        # TODO: Handle circular dependencies
        if ml_tool in ml_tool.parents:
            raise serializers.ValidationError({'parents': ['Tool {} depends on itself'.format(ml_tool)]})
        return ml_tool
