from rest_framework import permissions


class InternalGroupPermissionMixin(object):

    def has_permission(self, request, view):
        if request.user.is_authenticated and \
                not request.user.is_admin and \
                request.user.is_internal:
            return True

        return super().has_permission(request, view)


class VerifiedEmailPermissionMixin(object):

    def has_permission(self, request, view):
        if request.user.is_authenticated \
                and not request.user.is_admin \
                and not request.user.verified_email:
            # If the user has an unverified email, only allow HEAD, GET and OPTIONS.
            return super().has_permission(request, view) and request.method in permissions.SAFE_METHODS

        return super().has_permission(request, view)


class AllowAny(InternalGroupPermissionMixin, permissions.AllowAny):
    pass


class IsAuthenticated(InternalGroupPermissionMixin, permissions.IsAuthenticated):
    pass


class IsAuthenticatedOrReadOnly(InternalGroupPermissionMixin, permissions.IsAuthenticatedOrReadOnly):
    pass


class IsAdminUser(InternalGroupPermissionMixin, permissions.IsAdminUser):
    pass


class IsVerified(VerifiedEmailPermissionMixin, IsAuthenticated):
    pass


class IsVerifiedOrReadOnly(VerifiedEmailPermissionMixin, IsAuthenticatedOrReadOnly):
    pass
