from django.test import TestCase
from rest_framework.test import APITestCase
from arkindex.documents.models import Corpus
from arkindex.images.models import ImageServer
from arkindex.users.models import User


class FixtureMixin(object):
    """
    Add the database fixture to a test case
    """

    fixtures = ['data.json', ]

    @classmethod
    def setUpTestData(cls):
        cls.corpus = Corpus.objects.get(name='Unit Tests')
        cls.user = User.objects.get(email='user@user.fr')
        cls.superuser = User.objects.get(email='root@root.fr')
        cls.imgsrv = ImageServer.objects.get(url='http://server')


class FixtureTestCase(FixtureMixin, TestCase):
    """
    Django test case with the database fixture
    """


class FixtureAPITestCase(FixtureMixin, APITestCase):
    """
    Django REST Framework test case with the database fixture
    """
